from gi.repository import GObject, Gdk, GdkPixbuf, Gtk, Peas, RB

import rb

class SmallWindow (GObject.Object, Peas.Activatable):
	object = GObject.Property (type=GObject.Object)
	
	# Builder releated utility functions... ####################################
	
	def load_builder_content(self, builder):
		if( not hasattr( self, "__builder_obj_names" ) ):
			self.__builder_obj_names = list()
	
		for obj in builder.get_objects():
			if( isinstance( obj, Gtk.Buildable ) ):
				name = Gtk.Buildable.get_name(obj).replace(' ', '_')
				self.__dict__[ name ] = obj
				self.__builder_obj_names.append( name )

	def connect_builder_content( self, builder ):
		builder.connect_signals_full( self.connect_builder_content_func, self )

	def connect_builder_content_func( self,
									  builder,
									  object,
									  sig_name,
									  handler_name,
									  conn_object,
									  flags,
									  target ):
		handler = None
		
		h_name_internal = "_sh_" + handler_name.replace(" ", "_")
	
		if( hasattr( target, h_name_internal ) ):
			handler = getattr( target, h_name_internal )
		else:
			handler = eval(handler_name)
		
		object.connect( sig_name, handler )

	def purge_builder_content( self ):
		for name in self.__builder_obj_names:
			o = self.__dict__[ name ]
			if( isinstance( o, Gtk.Widget ) ):
				o.destroy()
			del self.__dict__[ name ]
	
		del self.__builder_obj_names
	
	
	
	# Plugins Methods... #######################################################
	
	def __init__(self):
		super(SmallWindow, self).__init__()
	
	def do_activate(self):
			# Basic Activation Procedure
		self.shell = self.object
		self.main_window = self.shell.props.window
		
			# Prepare internal variables
		self.song_duration = 0
		self.cover_pixbuf = None
		self.entry = None
		
			# Prepare Album Art Displaying
		self.album_art_db = GObject.new( RB.ExtDB, name="album-art" )
		
			# Build up small window interface
		builder = Gtk.Builder()
		builder.add_from_file( rb.find_plugin_file( self, "interface.ui" ) )
		self.load_builder_content( builder )
		self.connect_builder_content( builder )
		
			# Prepare windows
		for sub_widget in self.small_window:
			sub_widget.show_all()
		
		geometry = Gdk.Geometry()
		
		geometry.min_width = 300
		geometry.max_width = 5120
		geometry.min_height = -1
		geometry.max_height = -1
		
		self.small_window.set_geometry_hints( self.small_window,
			geometry,
			Gdk.WindowHints.MIN_SIZE | Gdk.WindowHints.MAX_SIZE )
		
			# Build up actions.
		self.ui_manager = self.shell.props.ui_manager
		
		self.ui_manager.insert_action_group( self.small_window_actions )
		self.ui_merge_id = self.ui_manager.add_ui_from_file( 
			rb.find_plugin_file( self,  "interface-ui.xml" ) )
		
			# Bring Builtin Actions to plugin
		main_actions_group = None
		for agroup in self.ui_manager.get_action_groups():
			if( agroup.get_name() == "MainActions" ):
				main_actions_group = agroup
				break
		
		for (a, b) in ((self.play_button, "ControlPlay"),
					   (self.prev_button, "ControlPrevious"),
					   (self.next_button, "ControlNext"),
					   (self.repeat_toggle, "ControlRepeat"),
					   (self.shuffle_toggle, "ControlShuffle")):
			a.set_related_action( main_actions_group.get_action( b ) )
		
			# Bind needed properites.
		self.bind_title = GObject.Binding(	source = self.main_window,
											source_property = "title",
											target = self.small_window,
											target_property = "title",
											flags = GObject.BindingFlags.DEFAULT )
		
			# Connect signal handlers to rhythmbox
		self.shell_player = self.shell.props.shell_player
		self.sh_psc = self.shell_player.connect("playing-song-changed",
												self._sh_on_song_change )
		
		self.sh_op = self.shell_player.connect("elapsed-changed",
												self._sh_on_playing )
	
	def do_deactivate(self):
		self.shell_player.disconnect( self.sh_op )
		self.shell_player.disconnect( self.sh_psc )
		del self.shell_player
		
		del self.bind_title
		
		self.ui_manager.remove_action_group( self.small_window_actions )
		self.ui_manager.remove_ui( self.ui_merge_id )
		
		del self.ui_merge_id
		del self.ui_manager
		
		del self.album_art_db
		
		self.purge_builder_content()
		
		del self.main_window
		del self.shell
	
	# Controlling Functions ####################################################
	
	def display_song(self, entry):
		self.entry = entry
		
		self.cover_pixbuf = None
		self.album_cover.clear()
		
		if( entry is None ):
			self.song_button_label.set_text( "" )
		
		else:
			self.song_button_label.set_markup(
				"<b>{title}</b> <small>{album} - {artist}</small>".format(
				title = entry.get_string( RB.RhythmDBPropType.TITLE ),
				album = entry.get_string( RB.RhythmDBPropType.ALBUM ),
				artist = entry.get_string( RB.RhythmDBPropType.ARTIST ) ) )
			
			key = entry.create_ext_db_key( RB.RhythmDBPropType.ALBUM )
			self.album_art_db.request( key,
									   self.display_song_album_art_callback,
									   entry )
	
	def display_song_album_art_callback( self, key, filename, data, entry ):
		if( ( data is not None ) and ( isinstance( data, GdkPixbuf.Pixbuf ) ) ):
			self.cover_pixbuf = data
			scale_cover = self.cover_pixbuf.scale_simple( 24, 24,
														  GdkPixbuf.InterpType.HYPER )
			
			self.album_cover.set_from_pixbuf( scale_cover )
		else:
			self.cover_pixbuf = None
			self.album_cover.clear()
	
	# Signal Handlers ##########################################################
	
	def _sh_small_window_action(self, object):
		self.main_window.hide()
		self.small_window.show()
	
	def _sh_main_window_action(self, object):
		self.small_window.hide()
		self.main_window.show()
	
	def _sh_small_window_on_close(self, window, asdf):
		self.shell.quit()
	
	def _sh_on_song_change(self, player, entry):
		if( entry is not None ):
			self.song_duration = entry.get_ulong( RB.RhythmDBPropType.DURATION )
		else:
			self.song_duration = 0
		self.display_song(entry)
	
	def _sh_on_playing(self, player, second ):
		if( self.song_duration != 0 ):
			self.song_progress.progress = float(second) / self.song_duration
		
	def _sh_progress_control( self, progress, fraction ):
		if( self.song_duration != 0 ):
			self.shell_player.set_playing_time( self.song_duration * fraction )
	
	def _sh_bigger_cover( self, cover, x, y, key, tooltip ):
		if( self.cover_pixbuf is not None ):
			tooltip.set_icon( self.cover_pixbuf )
			return True
		else:
			return False

################################################################################
# Custom Widgets ###############################################################

class SmallProgressBar( Gtk.DrawingArea ):
	
	__gsignals__ = {
		"control": (GObject.SIGNAL_RUN_LAST, None, (float,))
	}
	
	@GObject.Property
	def progress( self ):
		return self.__progress__
	
	@progress.setter
	def progress( self, value ):
		self.__progress__ = value
		self.queue_draw()
	
	def __init__( self ):
		super( SmallProgressBar, self).__init__()
		self.add_events( Gdk.EventMask.POINTER_MOTION_MASK |
						 Gdk.EventMask.BUTTON_PRESS_MASK |
						 Gdk.EventMask.BUTTON_RELEASE_MASK )
		self.button_pressed = False
		self.button_time = 0
		self.__progress__ = 0
	
	def do_draw( self, cc ):
		alloc = self.get_allocation()
		sc = self.get_style_context()
		fgc = sc.get_color( self.get_state_flags() )
		
		cc.set_source_rgba(1, 1, 1, 1 )
		cc.rectangle(0, 0, alloc.width, alloc.height )
		cc.fill()
		
		cc.set_source_rgba( fgc.red, fgc.green, fgc.blue, fgc.alpha )
		cc.rectangle(0, 0, alloc.width * self.progress, alloc.height )
		cc.fill()
		
	def do_motion_notify_event( self, event ):
		if( self.button_pressed ):
			self.control_by_event( event )
			return True
		else:
			return False
	
	def do_button_press_event( self, event ):
		self.button_pressed = True
		self.control_by_event( event )
		return True
	
	def do_button_release_event( self, event ):
		self.button_pressed = False
		self.control_by_event( event )
		return True
	
	def control_by_event( self, event ):
		allocw = self.get_allocated_width()
		fraction = event.x / allocw
		if( self.button_time + 100 < event.time ):
			self.button_time = event.time
			self.emit( "control", fraction )
